import { create } from "zustand";

type Paths = {
  audioDirectory: string;
  cacheDirectory: string;
  configDirectory: string;
  dataDirectory: string;
  desktopDirectory: string;
  documentDirectory: string;
  downloadDirectory: string;
  executableDirectory: string;
  fontDirectory: string;
  homeDirectory: string;
  logDirectory: string;
  pictureDirectory: string;
  templateDirectory: string;
  videoDirectory: string;
};

type PathStore = {
  paths: Paths;
  setPaths: () => Promise<void>;
};

const programTrailingPath = "stort/";

const initializePaths = async (): Promise<Paths> => {
  try {
    const {
      audioDir,
      cacheDir,
      configDir,
      dataDir,
      desktopDir,
      documentDir,
      downloadDir,
      executableDir,
      fontDir,
      homeDir,
      logDir,
      pictureDir,
      templateDir,
      videoDir,
    } = await import("@tauri-apps/api/path");

    return {
      audioDirectory: await audioDir(),
      cacheDirectory: (await cacheDir()) + programTrailingPath,
      configDirectory: (await configDir()) + programTrailingPath,
      dataDirectory: (await dataDir()) + programTrailingPath,
      desktopDirectory: await desktopDir(),
      documentDirectory: await documentDir(),
      downloadDirectory: await downloadDir(),
      executableDirectory: await executableDir(),
      fontDirectory: await fontDir(),
      homeDirectory: await homeDir(),
      logDirectory: await logDir(),
      pictureDirectory: await pictureDir(),
      templateDirectory: await templateDir(),
      videoDirectory: await videoDir(),
    };
  } catch (error) {
    console.error("Error initializing paths:", error);
    throw error;
  }
};

export const usePathStore = create<PathStore>((set) => ({
  paths: {
    audioDirectory: "",
    cacheDirectory: "",
    configDirectory: "",
    dataDirectory: "",
    desktopDirectory: "",
    documentDirectory: "",
    downloadDirectory: "",
    executableDirectory: "",
    fontDirectory: "",
    homeDirectory: "",
    logDirectory: "",
    pictureDirectory: "",
    templateDirectory: "",
    videoDirectory: "",
  },
  setPaths: async () => {
    try {
      const paths = await initializePaths();
      set({ paths });
    } catch (error) {
      console.error("Failed to set paths:", error);
    }
  },
}));
